from getch import getch
from textwrap import TextWrapper
import re
import sys
import os

use_color = True

ansi_trans = {
  '0': "\033[0m",     # reset all
  '1': "\033[31m",    # red
  '2': "\033[32m",    # green
  '3': "\033[33m",    # yellow
  '4': "\033[34m",    # blue
  '5': "\033[35m",    # magenta
  '6': "\033[36m",    # cyan
  '7': "\033[37m",    # white
  '8': "\033[1;30m",  # dark grey
  '9': "\033[1;31m",  # bright red
  'A': "\033[1;32m",  # bright green
  'B': "\033[1;33m",  # bright yellow
  'C': "\033[1;34m",  # bright blue
  'D': "\033[1;35m",  # bright magenta
  'E': "\033[1;36m",  # bright cyan
  'F': "\033[1;37m"   # bright white
}

def raw_write(outstr):
  sys.stdout.write(outstr)

def clear_screen():
  sys.stdout.write("\033[2J\033[0;0H")

def init():
  """ fiddle with this code if you're on windows and you want color to work """
  global use_color, clear_screen
  try:
    import msvcrt
    use_color = False
    def cscr():
      os.system('cls')
    clear_screen = cscr
  except ImportError:
    pass

def cwrite(outstr):
  wait_for_color = False
  for ch in outstr:
    if wait_for_color:
      if ch == '~':
        raw_write('~')
      else:
        if use_color:
          raw_write(ansi_trans[ch])
      wait_for_color = False
      continue
    if ch == '~':
      wait_for_color = True
    else:
      raw_write(ch)

def cflush():
  raw_write("\n")

def cwriteln(outstr):
  cwrite(outstr)
  cflush()

def hr(width=40):
  cwriteln("~4%s~0" % ('-' * width))

def cstrlen(outstr):
  count = 0
  wait_for_color = False
  for ch in outstr:
    if wait_for_color:
      if ch == '~':
        count += 1
      wait_for_color = False
      continue
    if ch == '~':
      wait_for_color = True
    else:
      count += 1
  return count

def wait_for_keypress():
  cwriteln("~6[press any key]~0")
  getch()
  cflush()

def menu(choices, header="Please choose:", show_char=False, display_only=False, width=40, wrap=True, select_multiple=False, prompt="[Esc] cancels"):

  selected = list()

  if select_multiple and prompt == "[Esc] cancels":
    prompt = "[Return] to accept, [Esc] cancels"

  while True:
    if select_multiple:
      clear_screen()
    # write header w/underline
    cwriteln(header)
    hr(cstrlen(header))
  
    # use textwrapper class, doesn't work well with color strings
    if wrap:
      txwp = TextWrapper(width=width)
      for cidx, choice in enumerate(choices):
        mark = '-'
        if cidx in selected:
          mark = '+'
        if show_char:
          txwp.initial_indent = "~6%s %s~0 " % (choice[0], mark)
          txwp.subsequent_indent = "    "
        else:
          txwp.initial_indent = "~6%s ~0" % mark
          txwp.subsequent_indent = "  "
        for line in txwp.wrap(choice[1]):
          cwriteln(line)

    # don't use textwrapper, depend on caller for formatting
    else:
      for cidx, choice in enumerate(choices):
        for index, line in enumerate(choice[1].split("\n")):
          line = line.strip()
          mark = '-'
          if cidx in selected:
            mark = '+'
          if show_char:
            if index == 0:
              cwriteln("~6%s %s~0 %s" % (choice[0], mark, line))
            else:
              cwriteln("    %s" % line)
          else:
            if index == 0:
              cwriteln("~6%s ~0%s" % (mark, line))
            else:
              cwriteln("  %s" % line)

    # return immediately if we were only called to display
    if display_only == True:
      return None

    if prompt:
      cwrite("%s ~4-~6-~0> " % prompt)
    else:
      cwrite("~4-~6-~0> ")

    while True:
      reprint = False
      ch = getch()
      if select_multiple:
        if ord(ch) == 13:
          cflush()
          return selected
      if ord(ch) == 27 or ord(ch) == 32:
        return None # cancelled
      for index, choice in enumerate(choices):
        if ch == choice[0]:
          if select_multiple:
            if index in selected:
              selected.remove(index)
            else:
              selected.append(index)
            reprint = True
          else:
            cwriteln(choice[0] + "~0")
            cflush()
            return index
      if reprint:
        break

def input_match_re(expr, prompt="Enter value: ", errstr="Invalid input!"):
  while True:
    cwrite(prompt)
    instr = raw_input()
    if instr == '':
      return None # user cancelled
    if expr.match(instr):
      return instr
    else:
      cwriteln(errstr)

def confirm(prompt="Confirm (y/n)? ", errstr="Enter 'y' or 'n', please."):
  val = input_match_re(re.compile(r"^[YyNn]$"), prompt, errstr)
  if val == 'y':
    return True
  else:
    return False

def text_join(elements):
  if len(elements) == 0:
    return ''
  if len(elements) == 1:
    return elements[0]
  pre = ', '.join(elements[0:-1])
  return ' and '.join([pre, elements[-1]])

if __name__ == '__main__':

  init()
  clear_screen()

  cwriteln(text_join(['one']))
  cwriteln(text_join(['one', 'two']))
  cwriteln(text_join(['one', 'two', 'three']))

  cwriteln("~1a~2b~3c~4d~5e~6f~7g~8h~9i~Aj~Bk~Cl~Dm~En~Fo~0~~")

  menu([['a', 'handgun'], ['b', 'Sed sollicitudin velit eu magna. Maecenas volutpat, nisl et dignissim pharetra, urna lectus ultrices est, vel pretium pede turpis id velit.'], ['c', 'dance shoes']],
    header="Your ~1inventory~0",
    show_char=False,
    display_only=True)

  cflush()
  choice = menu([['a', 'Morbi commodo, ipsum sed pharetra gravida, orci magna rhoncus neque, id pulvinar odio lorem non turpis.'], ['b', 'bar'], ['c', 'baz']], show_char=True)
  cwriteln(str(choice))
  cflush()
  
  cflush()
  choice = menu([['a', 'Morbi commodo, ipsum sed pharetra gravida, orci magna rhoncus neque, id pulvinar odio lorem non turpis.'], ['b', 'bar'], ['c', 'baz']], show_char=True, select_multiple=True)
  cwriteln(str(choice))
  cflush()
  
  cflush()
  choice = menu([['a', 'Morbi commodo, ipsum sed pharetr(a) gravida, orci magna rhoncus neque, id pulvinar odio lorem non turpis.'], ['b', '(b)ar'], ['c', '(c)az']], show_char=False, select_multiple=True, header="~~Yer stuff~~")
  cwriteln(str(choice))
  cflush()

  val = input_match_re(re.compile(r"^\d$"), prompt="Enter a single digit: ~6")
  cwriteln("~0got: %d" % int(val))

  val = input_match_re(re.compile(r"^\w{3,}$"), prompt="Enter your name: ~6",
    errstr="Three or more characters, please.")
  cwriteln("~0got: %s" % val)

  val = confirm(prompt="~1Really (y/n)? ~6")
  if val:
    cwriteln("~0yes, really!")
  else:
    cwriteln("~0nope")

